@@ -699,15 +699,15 @@ public:
 	VImage 
 	pow( double other, VOption *options = 0 )
 	{
-		return( math2_const( to_vector( other ), 
-			VIPS_OPERATION_MATH2_POW, options ) );
+		return( math2_const( VIPS_OPERATION_MATH2_POW, 
+			to_vector( other ), options ) );
 	}
 
 	VImage 
 	pow( std::vector<double> other, VOption *options = 0 )
 	{
-		return( math2_const( other, 
-			VIPS_OPERATION_MATH2_POW, options ) );
+		return( math2_const( VIPS_OPERATION_MATH2_POW, 
+			other, options ) );
 	}
 
 	VImage 
@@ -719,15 +719,15 @@ public:
 	VImage 
 	wop( double other, VOption *options = 0 )
 	{
-		return( math2_const( to_vector( other ), 
-			VIPS_OPERATION_MATH2_WOP, options ) );
+		return( math2_const( VIPS_OPERATION_MATH2_WOP, 
+			to_vector( other ), options ) );
 	}
 
 	VImage 
 	wop( std::vector<double> other, VOption *options = 0 )
 	{
-		return( math2_const( other, 
-			VIPS_OPERATION_MATH2_WOP, options ) );
+		return( math2_const( VIPS_OPERATION_MATH2_WOP, 
+			other, options ) );
 	}
 
 	VImage 
@@ -775,137 +775,137 @@ public:
 
 	std::vector<double> operator()( int x, int y );
 
-	friend VImage VIPS_CPLUSPLUS_API operator+( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator+( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator+( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator+( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator+( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator+( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator+( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator+( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator+( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator+( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator+=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator+=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator+=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator+=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator+=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator+=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator-( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator-( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator-( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator-( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator-( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator-( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator-( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator-( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator-( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator-( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator-=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator-=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator-=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator-=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator-=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator-=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator-( VImage a );
+	friend VIPS_CPLUSPLUS_API VImage operator-( VImage a );
 
-	friend VImage VIPS_CPLUSPLUS_API operator*( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator*( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator*( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator*( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator*( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator*( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator*( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator*( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator*( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator*( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator*=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator*=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator*=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator*=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator*=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator*=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator/( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator/( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator/( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator/( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator/( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator/( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator/( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator/( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator/( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator/( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator/=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator/=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator/=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator/=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator/=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator/=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator%( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator%( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator%( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator%( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator%( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator%( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator%=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator%=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator%=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator%=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator%=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator%=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator<( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator<( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator<( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator<( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<( VImage a, std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator<=( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<=( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<=( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator<=( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<=( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator<=( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<=( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<=( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator<=( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<=( VImage a, std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator>( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator>( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator>( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator>( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>( VImage a, std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator>=( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>=( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>=( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator>=( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>=( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator>=( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>=( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>=( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator>=( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>=( VImage a, std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator==( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator==( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator==( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator==( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator==( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator==( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator==( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator==( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator==( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator==( VImage a, std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator!=( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator!=( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator!=( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator!=( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator!=( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator!=( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator!=( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator!=( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator!=( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator!=( VImage a, std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator&( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator&( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator&( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator&( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator&( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator&( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator&( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator&( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator&( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator&( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator&=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator&=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator&=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator&=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator&=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator&=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator|( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator|( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator|( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator|( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator|( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator|( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator|( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator|( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator|( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator|( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator|=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator|=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator|=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator|=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator|=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator|=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator^( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator^( double a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator^( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator^( std::vector<double> a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator^( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator^( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator^( double a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator^( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator^( std::vector<double> a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator^( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator^=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator^=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator^=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator^=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator^=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator^=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator<<( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator<<( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator<<( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator<<( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator<<( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator<<( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator<<=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator<<=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator<<=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator<<=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator<<=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator<<=( VImage &a, const std::vector<double> b );
 
-	friend VImage VIPS_CPLUSPLUS_API operator>>( VImage a, VImage b );
-	friend VImage VIPS_CPLUSPLUS_API operator>>( VImage a, double b );
-	friend VImage VIPS_CPLUSPLUS_API operator>>( VImage a, std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage operator>>( VImage a, VImage b );
+	friend VIPS_CPLUSPLUS_API VImage operator>>( VImage a, double b );
+	friend VIPS_CPLUSPLUS_API VImage operator>>( VImage a, std::vector<double> b );
 
-	VImage & VIPS_CPLUSPLUS_API operator>>=( const VImage b );
-	VImage & VIPS_CPLUSPLUS_API operator>>=( const double b );
-	VImage & VIPS_CPLUSPLUS_API operator>>=( const std::vector<double> b );
+	friend VIPS_CPLUSPLUS_API VImage & operator>>=( VImage &a, const VImage b );
+	friend VIPS_CPLUSPLUS_API VImage & operator>>=( VImage &a, const double b );
+	friend VIPS_CPLUSPLUS_API VImage & operator>>=( VImage &a, const std::vector<double> b );
 
 };
 
