@@ -19,6 +19,10 @@
  * 	- redo as a class
  * 16/12/14
  * 	- free the input region much earlier
+ * 14/10/16
+ * 	- crop to a memory image rather than using a region ... this means we
+ * 	  use workers to calculate pixels and therefore use per-thread caching
+ * 	  in the revised buffer system
  */
 
 /*
@@ -86,9 +90,8 @@ vips_getpoint_build( VipsObject *object )
 {
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
 	VipsGetpoint *getpoint = (VipsGetpoint *) object;
+	VipsImage **t = (VipsImage **) vips_object_local_array( object, 2 );
 
-	VipsRect area; 
-	VipsRegion *region;
 	double *vector;
 	int n;
 	VipsArrayDouble *out_array;
@@ -96,31 +99,15 @@ vips_getpoint_build( VipsObject *object )
 	if( VIPS_OBJECT_CLASS( vips_getpoint_parent_class )->build( object ) )
 		return( -1 );
 
-	/* <0 ruled out already.
-	 */
-	if( getpoint->x >= getpoint->in->Xsize ||
-		getpoint->y >= getpoint->in->Ysize ) { 
-		vips_error( class->nickname, 
-			"%s", _( "coordinates out of range" ) ); 
-		return( -1 );
-	}
-
-	if( vips_check_coding_known( class->nickname, getpoint->in ) ||
-		!(region = vips_region_new( getpoint->in )) )
+	t[1] = vips_image_new_memory();
+	if( vips_crop( getpoint->in, &t[0], 
+		getpoint->x, getpoint->y, 1, 1, NULL ) ||
+		vips_image_write( t[0], t[1] ) )
 		return( -1 ); 
 
-	area.left = getpoint->x;
-	area.top = getpoint->y;
-	area.width = 1;
-	area.height = 1;
-	if( vips_region_prepare( region, &area ) ||
-		!(vector = vips__ink_to_vector( class->nickname, getpoint->in, 
-			VIPS_REGION_ADDR( region, area.left, area.top ), 
-			&n )) ) {
-		VIPS_UNREF( region );
+	if( !(vector = vips__ink_to_vector( class->nickname, 
+		getpoint->in, VIPS_IMAGE_ADDR( t[1], 0, 0 ), &n )) ) 
 		return( -1 );
-	}
-	VIPS_UNREF( region );
 
 	out_array = vips_array_double_new( vector, n );
 	g_object_set( object, 
@@ -131,11 +118,6 @@ vips_getpoint_build( VipsObject *object )
 	return( 0 );
 }
 
-/* xy range we sanity check on ... just to stop crazy numbers from 1/0 etc.
- * causing g_assert() failures later.
- */
-#define RANGE (100000000)
-
 static void
 vips_getpoint_class_init( VipsGetpointClass *class )
 {
@@ -167,14 +149,14 @@ vips_getpoint_class_init( VipsGetpointClass *class )
 		_( "Point to read" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsGetpoint, x ),
-		0, RANGE, 0 );
+		0, VIPS_MAX_COORD, 0 );
 
 	VIPS_ARG_INT( class, "y", 6, 
 		_( "y" ), 
 		_( "Point to read" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsGetpoint, y ),
-		0, RANGE, 0 );
+		0, VIPS_MAX_COORD, 0 );
 
 }
 
