@@ -23,6 +23,8 @@
  * 	- allow +/- INFINITY as a result
  * 4/12/12
  * 	- track and return top n values
+ * 24/1/17
+ * 	- sort equal values by y then x to make order more consistent
  */
 
 /*
@@ -143,10 +145,20 @@ vips_values_add( VipsValues *values, double v, int x, int y )
 
 	/* Find insertion point.
 	 */
-	for( i = 0; i < values->n; i++ )
-		if( v <= values->value[i] ) 
+	for( i = 0; i < values->n; i++ ) {
+		if( v < values->value[i] ) 
 			break;
 
+		if( v == values->value[i] ) {
+			if( y < values->y_pos[i] )
+				break;
+
+			if( y == values->y_pos[i] )
+				if( x <= values->x_pos[i] )
+					break;
+		}
+	}
+
 	/* Array full? 
 	 */
 	if( values->n == values->size ) {
@@ -498,6 +510,7 @@ vips_max_init( VipsMax *max )
  * By default it finds the single largest value. If @size is set >1, it will 
  * find the @size largest values. It will stop searching early if has found 
  * enough values. 
+ * Equal values will be sorted by y then x.
  *
  * It operates on all 
  * bands of the input image: use vips_stats() if you need to find an 
@@ -510,6 +523,9 @@ vips_max_init( VipsMax *max )
  * @out_array, @x_array and @y_array. These values are returned sorted from 
  * largest to smallest.
  *
+ * If there are more than @size maxima, the maxima returned will be a random
+ * selection of the maxima in the image. 
+ *
  * See also: vips_min(), vips_stats().
  *
  * Returns: 0 on success, -1 on error
