@@ -126,11 +126,9 @@ vips_cast_preeval( VipsImage *image, VipsProgress *progress, VipsCast *cast )
 static void
 vips_cast_posteval( VipsImage *image, VipsProgress *progress, VipsCast *cast )
 {
-	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( cast );
-
-	if( cast->overflow || cast->underflow ) 
-		vips_warn( class->nickname, 
-			_( "%d underflows and %d overflows detected" ),
+	if( cast->overflow || 
+		cast->underflow ) 
+		g_warning( _( "%d underflows and %d overflows detected" ),
 			cast->underflow, cast->overflow );
 }
 
@@ -544,7 +542,7 @@ vips_cast_class_init( VipsCastClass *class )
 	vobject_class->description = _( "cast an image" );
 	vobject_class->build = vips_cast_build;
 
-	operation_class->flags = VIPS_OPERATION_SEQUENTIAL_UNBUFFERED;
+	operation_class->flags = VIPS_OPERATION_SEQUENTIAL;
 
 	VIPS_ARG_IMAGE( class, "in", 1, 
 		_( "Input" ), 
