@@ -11,6 +11,8 @@
  * 	- support unicode on win
  * 19/8/16
  * 	- better transparency detection, thanks diegocsandrim
+ * 25/11/16
+ * 	- support @n, page-height
  */
 
 /*
@@ -82,8 +84,20 @@ typedef struct _VipsForeignLoadGif {
 	 */
 	int page;
 
+	/* Load this many pages.
+	 */
+	int n;
+
 	GifFileType *file;
 
+	/* The current read position, in pages.
+	 */
+	int current_page;
+
+	/* Set for EOF detected.
+	 */
+	gboolean eof;
+
 	/* As we scan the file, the index of the transparent pixel for this
 	 * frame.
 	 */
@@ -345,7 +359,6 @@ static void
 vips_foreign_load_gif_render_line( VipsForeignLoadGif *gif,
 	int width, VipsPel * restrict q, VipsPel * restrict p )
 {
-	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( gif );
 	ColorMapObject *map = gif->file->Image.ColorMap ?
 		gif->file->Image.ColorMap : gif->file->SColorMap;
 
@@ -355,8 +368,7 @@ vips_foreign_load_gif_render_line( VipsForeignLoadGif *gif,
 		VipsPel v = p[x];
 
 		if( v >= map->ColorCount ) {
-			vips_warn( class->nickname,
-				"%s", _( "pixel value out of range" ) );
+			g_warning( "%s", _( "pixel value out of range" ) );
 			continue;
 		}
 
@@ -414,6 +426,13 @@ vips_foreign_load_gif_render( VipsForeignLoadGif *gif, VipsImage *out )
 			}
 	}
 
+	/* We need a line buffer to decompress to.
+	 */
+	if( !gif->line ) 
+		if( !(gif->line = VIPS_ARRAY( gif, 
+			gif->file->SWidth, GifPixelType )) )
+			return( -1 ); 
+
 	if( file->Image.Interlace ) {
 		int i;
 
@@ -468,38 +487,17 @@ vips_foreign_load_gif_render( VipsForeignLoadGif *gif, VipsImage *out )
 	return( 0 );
 }
 
+/* Write the next page, if there is one, to @page. Set EOF if we hit the end of
+ * the file. @page must be a memory image of the right size. 
+ */
 static int
-vips_foreign_load_gif_to_memory( VipsForeignLoadGif *gif, VipsImage *out )
+vips_foreign_load_gif_page( VipsForeignLoadGif *gif, VipsImage *out )
 {
-	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( gif );
-
-	int frame_n;
 	GifRecordType record;
+	int n_pages;
 
-	vips_image_init_fields( out, 
-		gif->file->SWidth, gif->file->SHeight,
-		4, VIPS_FORMAT_UCHAR,
-		VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, 1.0, 1.0 );
+	n_pages = 0;
 
-	/* We will have the whole GIF frame in memory, so we can render any 
-	 * area.
-	 */
-        vips_image_pipelinev( out, VIPS_DEMAND_STYLE_ANY, NULL );
-
-	/* We need a line buffer to decompress to.
-	 */
-	gif->line = VIPS_ARRAY( gif, gif->file->SWidth, GifPixelType );
-
-	/* Turn out into a memory image which we then render the GIF frames
-	 * into.
-	 */
-	if( vips_image_write_prepare( out ) )
-		return( -1 );
-
-	/* Scan the GIF until we have enough to have completely rendered the
-	 * frame we need.
-	 */
-	frame_n = 0;
 	do { 
 		GifByteType *extension;
 		int ext_code;
@@ -521,9 +519,10 @@ vips_foreign_load_gif_to_memory( VipsForeignLoadGif *gif, VipsImage *out )
 			if( vips_foreign_load_gif_render( gif, out ) )
 				return( -1 ); 
 
-			frame_n += 1;
+			n_pages += 1;
 
-			VIPS_DEBUG_MSG( "gifload: start frame %d:\n", frame_n );
+			VIPS_DEBUG_MSG( "gifload: page %d:\n", 
+				gif->current_page + n_pages );
 
 			break;
 
@@ -533,7 +532,7 @@ vips_foreign_load_gif_to_memory( VipsForeignLoadGif *gif, VipsImage *out )
 			gif->transparency = -1;
 
 			if( DGifGetExtension( gif->file, 
-				&ext_code, &extension) == GIF_ERROR ) {
+				&ext_code, &extension ) == GIF_ERROR ) {
 				vips_foreign_load_gif_error( gif ); 
 				return( -1 ); 
 			}
@@ -572,6 +571,7 @@ vips_foreign_load_gif_to_memory( VipsForeignLoadGif *gif, VipsImage *out )
 
 		case TERMINATE_RECORD_TYPE:
 			VIPS_DEBUG_MSG( "gifload: TERMINATE_RECORD_TYPE:\n" ); 
+			gif->eof = TRUE;
 			break;
 
 		case SCREEN_DESC_RECORD_TYPE:
@@ -585,20 +585,170 @@ vips_foreign_load_gif_to_memory( VipsForeignLoadGif *gif, VipsImage *out )
 		default:
 			break;
 		}
-	} while( frame_n <= gif->page && 
-		record != TERMINATE_RECORD_TYPE );
+	} while( n_pages < 1 &&
+		!gif->eof );
 
-	if( frame_n <= gif->page ) {
+	gif->current_page += n_pages;
+
+	return( 0 );
+}
+
+static VipsImage *
+vips_foreign_load_gif_new_page( VipsForeignLoadGif *gif )
+{
+	VipsImage *out;
+
+	out = vips_image_new_memory();
+
+	vips_image_init_fields( out, 
+		gif->file->SWidth, gif->file->SHeight, 4, VIPS_FORMAT_UCHAR,
+		VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, 1.0, 1.0 );
+
+	/* We will have the whole GIF frame in memory, so we can render any 
+	 * area.
+	 */
+        vips_image_pipelinev( out, VIPS_DEMAND_STYLE_ANY, NULL );
+
+	/* Turn out into a memory image which we then render the GIF frames
+	 * into.
+	 */
+	if( vips_image_write_prepare( out ) ) {
+		g_object_unref( out ); 
+		return( NULL );
+	}
+
+	return( out );
+}
+
+static void *
+unref_object( void *data, void *a, void *b )
+{
+	g_object_unref( G_OBJECT( data ) );
+
+	return( NULL ); 
+}
+
+static void
+unref_array( GSList *list )
+{
+	/* g_slist_free_full() was added in 2.28 and we have to work with 
+	 * 2.6 :( 
+	 */
+	vips_slist_map2( list, unref_object, NULL, NULL ); 
+	g_slist_free( list );
+}
+
+/* We render each frame to a separate memory image held in a linked
+ * list, then assemble to out. We don't know the number of frames in advance,
+ * so we can't just allocate a large area.
+ */
+static int
+vips_foreign_load_gif_pages( VipsForeignLoadGif *gif, VipsImage **out )
+{
+	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( gif );
+
+	GSList *frames;
+	VipsImage *frame;
+	VipsImage *previous;
+	VipsImage **t;
+	int n_frames;
+	int i;
+
+	frames = NULL;
+	previous = NULL;
+
+	/* Accumulate any start stuff up to the first frame we need.
+	 */
+	if( !(frame = vips_foreign_load_gif_new_page( gif )) ) 
+		return( -1 );
+	do { 
+		if( vips_foreign_load_gif_page( gif, frame ) ) {
+			g_object_unref( frame );
+			return( -1 );
+		}
+	} while( !gif->eof &&
+		gif->current_page <= gif->page );
+
+	if( gif->eof ) {
+		vips_error( class->nickname, 
+			"%s", _( "too few frames in GIF file" ) );
+		g_object_unref( frame );
+		return( -1 );
+	}
+
+	frames = g_slist_append( frames, frame );
+	previous = frame;
+
+	while( gif->n == -1 ||
+		gif->current_page < gif->page + gif->n ) {
+		/* We might need a frame for this read to render to.
+		 */
+		if( !(frame = vips_foreign_load_gif_new_page( gif )) ) {
+			unref_array( frames );
+			return( -1 );
+		}
+
+		/* And init with the previous frame, if any.
+		 */
+		if( previous ) 
+			memcpy( VIPS_IMAGE_ADDR( frame, 0, 0 ),
+				VIPS_IMAGE_ADDR( previous, 0, 0 ),
+				VIPS_IMAGE_SIZEOF_IMAGE( frame ) );
+
+		if( vips_foreign_load_gif_page( gif, frame ) ) {
+			g_object_unref( frame ); 
+			unref_array( frames );
+			return( -1 );
+		}
+
+		if( gif->eof ) {
+			/* Nope, didn't need the new frame.
+			 */
+			g_object_unref( frame ); 
+			break;
+		}
+		else {
+			frames = g_slist_append( frames, frame );
+			previous = frame;
+		}
+	}
+
+	n_frames = g_slist_length( frames ); 
+
+	if( gif->eof &&
+		gif->n != -1 &&
+		n_frames < gif->n ) {
+		unref_array( frames );
 		vips_error( class->nickname, 
 			"%s", _( "too few frames in GIF file" ) );
 		return( -1 );
 	}
 
-	/* We've rendered to a memory image ... we can shut down the GIF
+	/* We've rendered to a set of memory images ... we can shut down the GIF
 	 * reader now.
 	 */
 	vips_foreign_load_gif_close( gif ); 
 
+	if( !(t = VIPS_ARRAY( gif, n_frames, VipsImage * )) ) { 
+		unref_array( frames );
+		return( -1 );
+	}
+
+	for( i = 0; i < n_frames; i++ )
+		t[i] = (VipsImage *) g_slist_nth_data( frames, i );
+
+	if( vips_arrayjoin( t, out, n_frames, 
+		"across", 1,
+		NULL ) ) { 
+		unref_array( frames );
+		return( -1 );
+	}
+
+	unref_array( frames );
+
+	if( n_frames > 1 )
+		vips_image_set_int( *out, VIPS_META_PAGE_HEIGHT, frame->Ysize );
+
 	return( 0 );
 }
 
@@ -611,11 +761,9 @@ vips_foreign_load_gif_load( VipsForeignLoad *load )
 
 	VipsImage *im;
 
-	/* Render to a memory image.
-	 */
-	im = t[0] = vips_image_new_memory();
-	if( vips_foreign_load_gif_to_memory( gif, im ) )
+	if( vips_foreign_load_gif_pages( gif, &t[0] ) )
 		return( -1 );
+	im = t[0];
 
 	/* Depending on what we found, transform and write to load->real.
 	 */
@@ -669,7 +817,7 @@ vips_foreign_load_gif_class_init( VipsForeignLoadGifClass *class )
 	gobject_class->set_property = vips_object_set_property;
 	gobject_class->get_property = vips_object_get_property;
 
-	object_class->nickname = "gifload";
+	object_class->nickname = "gifload_base";
 	object_class->description = _( "load GIF with giflib" );
 
 	load_class->get_flags_filename = 
@@ -683,11 +831,19 @@ vips_foreign_load_gif_class_init( VipsForeignLoadGifClass *class )
 		G_STRUCT_OFFSET( VipsForeignLoadGif, page ),
 		0, 100000, 0 );
 
+	VIPS_ARG_INT( class, "n", 6,
+		_( "n" ),
+		_( "Load this many pages" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignLoadGif, n ),
+		-1, 100000, 1 );
+
 }
 
 static void
 vips_foreign_load_gif_init( VipsForeignLoadGif *gif )
 {
+	gif->n = 1;
 	gif->transparency = -1;
 }
 
@@ -854,10 +1010,16 @@ vips_foreign_load_gif_buffer_init( VipsForeignLoadGifBuffer *buffer )
  * Optional arguments:
  *
  * * @page: %gint, page (frame) to read
+ * * @n: %gint, load this many pages
  *
  * Read a GIF file into a VIPS image.  Rendering uses the giflib library.
  *
- * Use @page to set page number (frame number) to read.
+ * Use @page to select a page to render, numbering from zero.
+ *
+ * Use @n to select the number of pages to render. The default is 1. Pages are
+ * rendered in a vertical column, with each individual page aligned to the
+ * left. Set to -1 to mean "until the end of the document". Use vips_grid() 
+ * to change page layout.
  *
  * The whole GIF is rendered into memory on header access. The output image
  * will be 1, 2, 3 or 4 bands depending on what the reader finds in the file. 
@@ -889,6 +1051,7 @@ vips_gifload( const char *filename, VipsImage **out, ... )
  * Optional arguments:
  *
  * * @page: %gint, page (frame) to read
+ * * @n: %gint, load this many pages
  *
  * Read a GIF-formatted memory block into a VIPS image. Exactly as
  * vips_gifload(), but read from a memory buffer. 
