@@ -48,7 +48,7 @@ extern "C" {
 typedef struct _VipsMeta {
 	VipsImage *im;
 
-	char *field;			/* strdup() of field name */
+	char *name;			/* strdup() of field name */
 	GValue value;			/* copy of value */
 } VipsMeta;
 
@@ -94,14 +94,11 @@ extern int vips__info;
  */
 extern char *vips__disc_threshold;
 
-/* Cache size settings.
- */
-extern char *vips__cache_max;
-extern char *vips__cache_max_mem;
-extern char *vips__cache_max_files;
 extern gboolean vips__cache_dump;
 extern gboolean vips__cache_trace;
 
+extern int vips__n_active_threads;
+
 void vips__threadpool_init( void );
 
 void vips__cache_init( void );
@@ -135,7 +132,6 @@ int vips_mapfilerw( VipsImage * );
 int vips_remapfilerw( VipsImage * );
 
 void vips__buffer_init( void );
-void vips__buffer_shutdown( void );
 
 void vips__copy_4byte( int swap, unsigned char *to, unsigned char *from );
 void vips__copy_2byte( gboolean swap, unsigned char *to, unsigned char *from );
@@ -231,7 +227,7 @@ int vips_check_bands_3ormore( const char *domain, VipsImage *im );
 
 int vips__byteswap_bool( VipsImage *in, VipsImage **out, gboolean swap );
 
-char *vips__make_xml_metadata( const char *domain, VipsImage *image );
+char *vips__xml_properties( VipsImage *image );
 
 void vips__cairo2rgba( guint32 *buf, int n );
 
@@ -254,6 +250,9 @@ int vips__foreign_convert_saveable( VipsImage *in, VipsImage **ready,
 
 int vips__image_intize( VipsImage *in, VipsImage **out );
 
+void vips__reorder_init( void );
+int vips__reorder_set_input( VipsImage *image, VipsImage **in );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
