@@ -46,6 +46,7 @@
 
 /*
 #define VIPS_DEBUG
+#define DEBUG
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,9 +68,6 @@
 
 /* Set by GOption from the command line, eg. "12m".
  */
-char *vips__cache_max = NULL;
-char *vips__cache_max_mem = NULL;
-char *vips__cache_max_files = NULL;
 gboolean vips__cache_dump = FALSE;
 gboolean vips__cache_trace = FALSE;
 
@@ -456,18 +454,6 @@ vips__cache_init( void )
 		vips_cache_table = g_hash_table_new( 
 			(GHashFunc) vips_operation_hash, 
 			(GEqualFunc) vips_operation_equal );
-
-		if( vips__cache_max ) 
-			vips_cache_max = 
-				vips__parse_size( vips__cache_max );
-
-		if( vips__cache_max_mem ) 
-			vips_cache_max_mem = 
-				vips__parse_size( vips__cache_max_mem );
-
-		if( vips__cache_max_files ) 
-			vips_cache_max_files = 
-				vips__parse_size( vips__cache_max_files );
 	}
 }
 
@@ -549,6 +535,10 @@ vips_cache_remove( VipsOperation *operation )
 	VipsOperationCacheEntry *entry = (VipsOperationCacheEntry *)
 		g_hash_table_lookup( vips_cache_table, operation );
 
+#ifdef DEBUG
+	printf( "vips_cache_remove: trimming %p\n", operation );
+#endif /*DEBUG*/
+
 	g_assert( entry ); 
 
 	if( entry->invalidate_id ) { 
