@@ -10,11 +10,11 @@ set -e
 
 # poppler / pdfload reference image
 poppler=$test_images/blankpage.pdf
-poppler_ref=$test_images/blankpage.png
+poppler_ref=$test_images/blankpage.pdf.png
 
 # rsvg / svgload reference image
 rsvg=$test_images/blankpage.svg
-rsvg_ref=$test_images/blankpage.png
+rsvg_ref=$test_images/blankpage.svg.png
 
 # giflib / gifload reference image
 giflib=$test_images/trans-x.gif
@@ -49,7 +49,8 @@ save_load() {
 	fi
 
 	if ! $vips copy $tmp/t1.$format $tmp/back.v ; then
-		echo "read from $out failed"
+		echo "read from $tmp/t1.format failed"
+		echo "  (was written by $vips copy $in $tmp/t1.$format$mode)"
 		exit 1
 	fi
 }
@@ -138,13 +139,14 @@ test_loader() {
 	ref=$1
 	in=$2
 	format=$3
+	thresh=$4
 
 	printf "testing $(basename $in) $format ... "
 
 	$vips copy $ref $tmp/before.v
 	$vips copy $in $tmp/after.v
 
-	test_difference $tmp/before.v $tmp/after.v 0
+	test_difference $tmp/before.v $tmp/after.v $thresh
 
 	echo "ok"
 }
@@ -228,19 +230,20 @@ test_raw $mono
 test_raw $image 
 
 if test_supported pdfload; then
-	test_loader $poppler_ref $poppler pdfload
+	test_loader $poppler_ref $poppler pdfload 0
 fi
 
 if test_supported svgload; then
-	test_loader $rsvg_ref $rsvg svgload
+	# librsvg can give small differences on some platforms
+	test_loader $rsvg_ref $rsvg svgload 10
 fi
 
 if test_supported gifload; then
-	test_loader $giflib_ref $giflib gifload
+	test_loader $giflib_ref $giflib gifload 0
 fi
 
 if test_supported matload; then
-	test_loader $matlab_ref $matlab matlab
+	test_loader $matlab_ref $matlab matlab 0
 fi
 
 if test_supported dzsave; then
